@@ -12,45 +12,22 @@
 #   with the exception that it cannot be placed on a CD-ROM or similar media
 #   for commercial distribution without the prior approval of the author.
 #------------------------------------------------------------------------------
-eval 'exec /usr2/local/bin/perl -S $0 ${1+"$@"}'
-  if 0;
 
 package Math::Currency;
+$Math::Currency::VERSION = '0.50';
+# ABSTRACT: Exact Currency Math with Formatting and Rounding
 
 use strict;
 use utf8;
-use version;
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $PACKAGE $FORMAT $LC_MONETARY
-  $accuracy $precision $div_scale $round_mode $use_int $always_init);
-use Exporter;
+use base qw(Exporter Math::BigFloat);
 use Math::BigFloat 1.60;
-use overload '""' => \&bstr;
 use POSIX qw(locale_h);
 use Encode::Locale;
 use Encode ();
 
-use constant _LEGACY_MATH_BIGINT => 
-    (version->parse(Math::BigInt->VERSION) <= version->parse('1.999717'));
+use overload '""' => \&bstr;
 
-@ISA = qw(Exporter Math::BigFloat);
-
-# Items to export into callers namespace by default. Note: do not export
-# names by default without a very good reason. Use EXPORT_OK instead.
-# Do not simply export all your public functions/methods/constants.
-@EXPORT = qw(
-);
-
-@EXPORT_OK = qw(
-  $LC_MONETARY
-  $FORMAT
-  Money
-);
-
-$VERSION = 0.49;
-
-$PACKAGE = __PACKAGE__;
-
-$LC_MONETARY = {
+our $LC_MONETARY = {
     en_US => {
         INT_CURR_SYMBOL   => 'USD ',
         CURRENCY_SYMBOL   => '$',
@@ -71,25 +48,24 @@ $LC_MONETARY = {
 };
 $LC_MONETARY->{USD} = $LC_MONETARY->{en_US};
 
-unless ( localize() )    # no locale information available
-{
-    $FORMAT = $LC_MONETARY->{en_US};
-}
+our $FORMAT = $LC_MONETARY->{en_US} unless localize();
+
+our @EXPORT_OK = qw(
+    $LC_MONETARY
+    $FORMAT
+    Money
+);
 
 # Set class constants
-$round_mode = 'even';    # Banker's rounding obviously
-$accuracy   = undef;
-$precision = $FORMAT->{FRAC_DIGITS} > 0 ? -$FORMAT->{FRAC_DIGITS} : 0;
-$div_scale = 40;
-$use_int   = 0;
-$always_init = 0;        # should the localize() happen every time?
+our $round_mode = 'even';    # Banker's rounding obviously
+our $accuracy   = undef;
+our $precision = $FORMAT->{FRAC_DIGITS} > 0 ? -$FORMAT->{FRAC_DIGITS} : 0;
+our $div_scale = 40;
+our $use_int   = 0;
+our $always_init = 0;        # should the localize() happen every time?
 
-# Preloaded methods go here.
-############################################################################
-sub new                  #05/10/99 3:13:PM
-############################################################################
 
-{
+sub new {
     my $proto  = shift;
     my $class  = ref($proto) || $proto;
     my $parent = $proto if ref($proto);
@@ -111,10 +87,9 @@ sub new                  #05/10/99 3:13:PM
     my $currency = shift;
     my $format;
 
-    if ( not defined $currency and $class->isa($PACKAGE) ) {
-
+    if ( not defined $currency and $class->isa('Math::Currency') ) {
         # must be one of our subclasses
-        $currency = $1 if ($class =~ /$PACKAGE\:\:(\w+)/);
+        $currency = $1 if ($class =~ /Math::Currency::(\w+)/);
     }
 
     if ( defined $currency )    #override default currency type
@@ -149,19 +124,12 @@ sub new                  #05/10/99 3:13:PM
     return $self;
 }    ##new
 
-############################################################################
-sub Money    #05/10/99 4:16:PM
-############################################################################
 
-{
-    return $PACKAGE->new(@_);
-}            ##Money
+sub Money {
+    return __PACKAGE__->new(@_);
+}
 
-############################################################################
-sub bstr     #05/10/99 3:52:PM
-############################################################################
-
-{
+sub bstr {
     my $self     = shift;
     my $myformat = $self->format();
     my $value    = $self->as_float();
@@ -228,11 +196,9 @@ s/(\d{$myformat->{MON_GROUPING}})(?=\d)(?!\d*\.)/$1$myformat->{MON_THOUSANDS_SEP
     return $value;
 }    ##stringify
 
-############################################################################
-sub format    #05/17/99 1:58:PM
-############################################################################
 
-{
+
+sub format {
     my $self  = shift;
     my $key   = shift;    # do they want to display or set?
     my $value = shift;    # did they supply a value?
@@ -288,6 +254,7 @@ sub format    #05/17/99 1:58:PM
     return $$source;
 }    ##format
 
+
 sub as_float {
     my $self   = shift;
     my $format = $self->format;
@@ -295,6 +262,7 @@ sub as_float {
     return $string;
 }
 
+
 sub copy {
     my $self = shift;
 
@@ -314,6 +282,8 @@ sub copy {
     return $new;
 }
 
+
+
 sub as_int {
     my $self = shift;
     (my $str = $self->as_float) =~ s/\.//o;
@@ -324,20 +294,25 @@ sub as_int {
 # we override the default here because we only want to compare the precision of
 # the currency we're dealing with, not the precision of the underlying object
 sub bcmp {
-    # must use package for Math::BigInt >= 1.999717. See RT #115247
-    my $class = _LEGACY_MATH_BIGINT ? shift : __PACKAGE__;
+    # See RT #115247, #115761
+    # bcmp() might get called in comparison overoad as an object method with
+    # one arg, or, as a class method with two args depending on the version of
+    # Math::BigInt that is installed. Workaround is to check if @_ has three
+    # args, and if so, the first one is the class name.  Otherwise, the first
+    # arg is the left side is an object that bcmp() is called on.
+    # An alternate solution to this is to require Math::BigInt >= 1.999718
+    # which always uses bcmp() as an object method.
+    my $class = (@_ == 3) ? shift : __PACKAGE__;
 
     # make sure we're dealing with two Math::Currency objects
     my ( $x, $y ) =
-      map { ref $_ ne $class ? $class->new($_) : $_ } @_[ 0, 1 ];
+        map { ref $_ ne $class ? $class->new($_) : $_ } @_[ 0, 1 ];
+
     return $x->as_float <=> $y->as_float;
 }
 
-############################################################################
-sub localize    #08/17/02 7:58:PM
-############################################################################
 
-{
+sub localize {
     my $self   = shift;
     my $format = shift || \$FORMAT;
 
@@ -388,9 +363,12 @@ sub localize    #08/17/02 7:58:PM
     return 1;
 }
 
+
 {
     my $locales;
     sub available_locales {
+        return if $^O =~ /Win32/; # cant run locale -a on windows
+
         unless (defined $locales) {
             $locales = [];
 
@@ -408,20 +386,17 @@ sub localize    #08/17/02 7:58:PM
     }
 }
 
-############################################################################
-sub unknown_currency    #02/03/05 4:37am
-############################################################################
-
-{
+# if no currency module exists for the requested currency, this function tries
+# to find the currency settings from the available locales
+sub unknown_currency {
     my ($currency) = @_;
-    $DB::single=1;
 
     # remember current locale
     my $original_locale = setlocale( LC_ALL );
 
     # we need to save a copy of $VERSION here becuase the effective locale can
     # render $VERSION as X,YY instead of Y.YY for exmaple
-    my $version = "$VERSION";
+    my $version = "$Math::Currency::VERSION";
 
     for my $LOCALE (available_locales()) {
         setlocale( LC_ALL, $LOCALE );
@@ -436,18 +411,20 @@ sub unknown_currency    #02/03/05 4:37am
                 unless exists $LC_MONETARY->{$int_curr};
             eval <<"EOP";
 package Math::Currency::${LOCALE};
-use vars qw(\$VERSION \@ISA \$LANG);
 
-\$VERSION = $version;
-\$LANG  = '$LOCALE';
-\@ISA = qw/Math::Currency/;
+use base 'Math::Currency';
+our \$VERSION = $version;
+our \$LANG  = '$LOCALE';
+
 1;
-package Math::Currency::${int_curr};
-use vars qw(\$VERSION \@ISA \$LANG);
 
-\$VERSION = $version;
-\$LANG  = '$LOCALE';
-\@ISA = qw/Math::Currency/;
+package Math::Currency::${int_curr};
+
+use base 'Math::Currency';
+
+our \$VERSION = $version;
+our \$LANG  = '$LOCALE';
+
 1;
 EOP
             last;
@@ -460,36 +437,55 @@ EOP
 
 # additional methods needed to get/set package globals
 
+
 sub always_init {
-    my ($class) = shift;
-    $always_init = shift if @_;
+    my $class = shift;
+
+    if (@_) {
+        $always_init = shift;
+    }
+
     return $always_init;
 }
 
+
 sub use_int {
-    my ($class) = shift;
-    $use_int = shift if @_;
+    my $class = shift;
+
+    if (@_) {
+        $use_int = shift;
+    }
+
     return $use_int;
 }
 
-# Autoload methods go after =cut, and are processed by the autosplit program.
-
 1;
+
 __END__
 
+=pod
+
 =head1 NAME
 
 Math::Currency - Exact Currency Math with Formatting and Rounding
 
+=head1 VERSION
+
+version 0.50
+
 =head1 SYNOPSIS
 
  use Math::Currency qw(Money $LC_MONETARY);
- $dollar = Math::Currency->new("$12,345.67");
- $taxamt = $dollar * 0.28;
+
+ my $dollar = Math::Currency->new("$12,345.67");
+ my $taxamt = $dollar * 0.28;
+
  # this sets the default format for all objects w/o their own format
  Math::Currency->format('EUR');
- $euro = Money(12345.67);
- $euro_string = Money(12345.67)->bstr();
+
+ my $euro = Money(12345.67);
+ my $euro_string = Money(12345.67)->bstr();
+
  # or if you already have a Math::Currency object
  $euro_string = "$euro";
 
@@ -513,6 +509,166 @@ currency variable, you can treat it like any number and the module will do
 the right thing.  This module is a thin layer over Math::BigFloat which
 is itself a layer over Math::BigInt.
 
+=head1 METHODS
+
+=head2 new
+
+Create a new currency object.  This can be called a number of ways:
+
+=over 4
+
+=item *
+
+new()
+
+Creates a new currency object with value "0".
+
+=item *
+
+new($value)
+
+Creates a new currency object with the given value. Note that C<$value> should
+be quoted to avoid locale formatting issues.  E.g.: C<< new('1234.56') >>.
+
+=item *
+
+new($value, $curency)
+
+Creates a new currency object in the given locale.  See L<Predefined Locales>.
+
+=back
+
+=head2 format($key, $value)
+
+Set object or global formatting.  See L<Object Formats> for more information.
+
+=head2 as_float
+
+Bare floating point notation without currency formatting.
+
+When storing the value into a database, you often need a string which
+corresponds to the value of the currency as a floating point number, but
+without the special currency formatting.  That is what this object method
+produces.  Be sure and use e.g. DECIMAL(10,2) in MySQL, or NUMERIC(10,3) in
+PostgreSQL to ensure that you don't have any floating point rounding issues
+going from/to the database.
+
+=head2 copy
+
+Returns a copy of the current Math::Currency object.
+
+=head2 as_int
+
+Some US credit card gateways require all transactions to be expressed in
+pennies.  This object method returns an integer value that corresponds to the
+currency value multiplied by 10 to the power of the number of decimal places of
+precision.  Essentially, this expresses the currency amount in the smallest
+discrete value allowed with that currency, so for currency expressed in 
+dollars, this method returns the same value in pennies.
+
+=head2 localize
+
+Reinitialize the global (if called with no arguments), or local (if called with
+a reference to a hashref) format hash.
+
+If you don't want to always have to remember to reinitialize the POSIX settings
+when you switch locales, see L<always_init>
+
+Examples:
+
+=over 4
+
+=item *
+
+C<< Math::Currency->localize() >>
+
+Renitialize the global internal format hash using the current locale
+
+=item *
+
+C<< Math::Currency->localize(\$format) >>
+
+Renitialize the hashref C<$format> using the current locale.
+
+=back
+
+=head2 available_locales
+
+Get the list of available locales on this system
+
+=head2 always_init
+
+If you don't want to always have to remember to reinitialize the POSIX settings
+when you switch locales, you can call this with a true value.
+
+ Math::Currency->always_init(1);
+
+Alternatively, you can set the global variable:
+
+ $Math::Currency::always_init = 1;
+
+and every single time a Math::Currency object is printed, the global $FORMAT
+will be updated to the locale current at that time.  This may be a performance
+hit.  It would be better if you followed the method of manually updating the
+global format immediately after you reset the locale.
+
+=head3 Notes
+
+=over 4
+
+=item *
+
+This function will reset only the global format and will not have effect on
+objects created with their own overridden formats, even if they were
+originally based on the global format.
+
+=item *
+
+You must have all the locale files in question already loaded; the list
+reported by C<locale -a> is not always a reliable judge of what files you might
+actually have installed.  If you try and set a nonexistant locale, or set the
+same locale as is already active, the module will silently retain the current
+locale settings.
+
+=back
+
+=head2 use_int
+
+Default: false
+
+Pass a true value to this to indicate that the international currency symbol
+(C<INT_CURR_SYMBOL>) should be used instead of local currency symbol
+(C<CURRENCY_SYMBOL>).  Note that this is a global setting for the library and
+will affect all objects.  For example, in the C<USD> locale, stringification of
+C<1234.56> looks like:
+
+=over 4
+
+=item *
+
+use_int(0)
+
+$1,234.56
+
+=item *
+
+use_int(1)
+
+USD 1,234.56
+
+=back
+
+=head1 FUNCTIONS
+
+=head2 Money
+
+This an exportable function that constructs a new object.  Takes the same arguments
+as C<new()>.
+
+=encoding UTF-8
+
+=for Pod::Coverage unknown_currency
+
 =head1 Important Note on Input Values
 
 Since the point of this module is to perform currency math and not floating
@@ -582,13 +738,6 @@ unsupported locales).  If you are using a locale in a country that uses the
 Euro, you should create your own local format file using your default LANG 
 setting, since the Euro formatting rules are country specific.
 
-B<IMPORTANT NOTE>: the predefined locales have been generated using non-UTF-8
-locales (since all of the above currencies have a native ASCII character
-available).  If you use a UTF-8 locale (common with more modern Linux distros),
-then the non-UTF-8 version will be used if found.  Some locales require the
-use of UTF-8 to represent their L<local currency|Currency Symbol>, so you must
-generate your own L<Custom Locale> to fully support that usage.
-
 If you want to use any locale other than your default in a single script, there
 are two different ways to specify which currency format you wish to use, with
 somewhat subtle differences:
@@ -607,7 +756,7 @@ program, use this mode:
 The last line above will automatically load the applicable subclass and
 use that formatting for that specific object.  These formats can either use
 a pre-generated subclass or will automatically generate an automatic
-L<Custom Locale>, 
+L<Custom Locale>,
 
 =back
 
@@ -627,7 +776,7 @@ rules, create the objects directly using the subclass:
 =head2 Currency Symbol
 
 The locale definition includes two different Currency Symbol strings: one
-is the native character(s), like $ or  or ; the other is the three
+is the native character(s), like $ or £ or ¥; the other is the three
 character string defined by the ISO4217 specification followed by the
 normal currency separation character (frequently space).  The default
 behavior is to always display the native CURRENCY_SYMBOL unless a global
@@ -699,24 +848,7 @@ at any time by using:
     Math::Currency->localize;    # reinitialize global format
 
 If you don't want to always have to remember to reinitialize the POSIX settings
-when you switch locales, you can set the global parameter:
-
-    $Math::Currency::always_init = 1;
-
-and every single time a M::C object is printed, the global $FORMAT will be
-updated to the locale current at that time.  This may be a performance hit.  It
-would be better if you followed the first method of manually updating the global
-format immediately after you reset the locale.
-
-NOTE: This function will reset only the global format and will not have
-effect on objects created with their own overridden formats, even if they were
-originally based on the global format.
-
-NOTE 2: You must have all the locale files in question already loaded; the list
-reported by `locale -a` is not always a reliable judge of what files you
-might actually have installed.  If you try and set a nonexistant locale,
-or set the same locale as is already active, the module will silently retain
-the current locale settings.
+when you switch locales, see L<always_init>
 
 =head2 Object Formats
 
@@ -790,49 +922,53 @@ parameters:
 
 (the negative variants are similar).
 
-=head2 Additional Object Methods
-
-There are times when you would like to take a Math::Currency object and use
-it with some other module or external agent which doesn't understand the
-currency formatting.  
+=head1 SEE ALSO
 
 =over 4
 
-=item $m->as_float - bare floating point notation without currency formatting
+=item *
 
-When storing the value into a database, you often need a string which
-corresponds to the value of the currency as a floating point number, but
-without the special currency formatting.  That is what this object method
-produces.  Be sure and use e.g. DECIMAL(10,2) in MySQL, to ensure that you
-don't have any floating point rounding issues going from/to the database.
+perllocale
 
-=item $m->as_int - bare integer number of "minimum value" 
+=item *
 
-Some US credit card gateways require all transactions to be expressed in
-pennies (because their software isn't running Math::Currency!).  This
-object method returns an integer value that corresponds to the currency
-value multiplied by 10 to the power of the number of decimal places of
-precision.  Essentially, this expresses the currency amount in the smallest
-discrete value allowed with that currency, so for currency expressed in 
-dollars, this method returns the same value in pennies.
+Math::BigFloat
+
+=item *
+
+Math::BigInt
 
 =back
 
+=head1 SOURCE
+
+The development version is on bitbucket at
+https://bitbucket.org/jpeacock/math-currency
+
 =head1 BUGS
 
-Please report any bugs or feature requests to
-C<bug-Math-Currency@rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org>.
+Please report any bugs or feature requests to bug-math-currency@rt.cpan.org or through the web interface at:
+ http://rt.cpan.org/Public/Dist/Display.html?Name=Math-Currency
 
-=head1 AUTHOR
+=head1 AUTHORS
+
+=over 4
+
+=item *
 
 John Peacock <jpeacock@cpan.org>
 
-=head1 SEE ALSO
+=item *
 
- perl(1).
- perllocale
- Math::BigFloat
- Math::BigInt
+Michael Schout <mschout@gkg.net>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2001 by John Peacock <jpeacock@cpan.org>.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
